/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * main.c - demo main module to test irmp decoder
 *
 * Copyright (c) 2009-2010 Frank Meyer - frank(at)fli4l.de
 *
 * ATMEGA88 @ 8 MHz
 *
 * Fuses: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "irmp.h"


#define F_CPU 8000000UL
#ifndef F_CPU
#error F_CPU unkown
#endif

#define F_INTERRUPT   10000                                                     // 10000 interrupts per second


/*-----------------------------------------------------------------------------------------------------------------------
 * UART routines
 *-----------------------------------------------------------------------------------------------------------------------
 */
#define BAUD                                    9600L
#include <util/setbaud.h>

#ifdef UBRR0H

#define UART0_UBRRH                             UBRR0H
#define UART0_UBRRL                             UBRR0L
#define UART0_UCSRA                             UCSR0A
#define UART0_UCSRB                             UCSR0B
#define UART0_UCSRC                             UCSR0C
#define UART0_UDRE_BIT_VALUE                    (1<<UDRE0)
#define UART0_UCSZ1_BIT_VALUE                   (1<<UCSZ01)
#define UART0_UCSZ0_BIT_VALUE                   (1<<UCSZ00)
#ifdef URSEL0
#define UART0_URSEL_BIT_VALUE                   (1<<URSEL0)
#else
#define UART0_URSEL_BIT_VALUE                   (0)
#endif
#define UART0_TXEN_BIT_VALUE                    (1<<TXEN0)
#define UART0_RXEN_BIT_VALUE                    (1<<RXEN0)
#define UART0_UDR                               UDR0
#define UART0_U2X                               U2X0
#define UART0_RXC                               RXC0

#else

#define UART0_UBRRH                             UBRRH
#define UART0_UBRRL                             UBRRL
#define UART0_UCSRA                             UCSRA
#define UART0_UCSRB                             UCSRB
#define UART0_UCSRC                             UCSRC
#define UART0_UDRE_BIT_VALUE                    (1<<UDRE)
#define UART0_UCSZ1_BIT_VALUE                   (1<<UCSZ1)
#define UART0_UCSZ0_BIT_VALUE                   (1<<UCSZ0)

#ifdef URSEL
#define UART0_URSEL_BIT_VALUE                   (1<<URSEL)
#else
#define UART0_URSEL_BIT_VALUE                   (0)
#endif
#define UART0_TXEN_BIT_VALUE                    (1<<TXEN)
#define UART0_RXEN_BIT_VALUE                    (1<<RXEN)
#define UART0_UDR                               UDR
#define UART0_U2X                               U2X
#define UART0_RXC                               RXC

#endif

/*
per Programm ermittelte Codes
Adress 0x46
commands
0x1 LW1 /FM1
0x2 LW2 /FM2
0x3 LW2 /FM3
0x4 MW4 /FM4
0x5 MW5 /FM5
0x6 MW6 /FM6
0xa Suchlauf -
0xb Suchlauf +
0xc ? schaltet auf MW
0xd wechselt FM Bereiche 1-3
0xf numerische Eingabe

0x12 Funktions Anwahl (Direkt)
0x13 Funktions wechsel
0x14 Vol +
0x15 Vol -
0x16 ATT on / off
0x17 Audio Control
0x1C on/off
0x1E BT Umschaltung (Audio / USB)
0x92 BT Umschaltung (Audio / USB)
0xd8 Display Test? schaltet alle Digits im Display ein
0xd9 Reset


Codes von FB ausgelesen

* Adresse           0x9D
* Taste 1           0x80
* Taste 2           0x40
* Taste 3           0xc0
* Taste 4           0x20
* Taste 5           0xa0
* Taste 6           0x60
* Taste 7           0xe0
* Taste 8           0x10
* Taste 9           0x90
* Taste 0           0x00
* tel               0x49
* direct            0xf0
* AM-               0x30
* FM+               0xb0
* exit              0x71
* AUD               0xe8
* down              0x61
* up                0xe1
* left              0x50
* right             0xd0
* ENTER/Play/Pause  0x70
* ATT               0x68
* Return/Repeat     0xa1
* Vol-              0xa8
* Vol+              0x28
* Src               0xC8
*/




/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Initialize  UART
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
static void
uart_init (void)
{
    UART0_UBRRH = UBRRH_VALUE;                                                                      // set baud rate
 	UART0_UBRRL = UBRRL_VALUE;
	
#if USE_2X
    UART0_UCSRA |= (1<<UART0_U2X);
#else
    UART0_UCSRA &= ~(1<<UART0_U2X);
#endif

    UART0_UCSRC = UART0_UCSZ1_BIT_VALUE | UART0_UCSZ0_BIT_VALUE | UART0_URSEL_BIT_VALUE;
    UART0_UCSRB |= UART0_TXEN_BIT_VALUE;                                                            // enable UART TX
    UART0_UCSRB |= UART0_RXEN_BIT_VALUE;                                                            // activate UART RX
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Send character
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
static void
uart_putc (unsigned char ch)
{
    while (!(UART0_UCSRA & UART0_UDRE_BIT_VALUE))
    {
        ;
    }

    UART0_UDR = ch;
}


/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Send string
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
static void
uart_puts (char * s)
{
    volatile int i=0;
	while (*s)
    {
        uart_putc (*s++);
		i++;
    }
}


void timer_init (void)
{
  OCR1A   = (F_CPU / F_INTERRUPT) - 1;                                          // compare value: 1/10000 of CPU frequency
  TCCR1B  = (1 << WGM12) | (1 << CS10);                                         // switch CTC Mode on, set prescaler to 1
  TIMSK   = 1 << OCIE1A;                                                        // OCIE1A: Interrupt by timer compare (use TIMSK for ATMEGA162)
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
ISR(TIMER1_COMPA_vect)
{
  irmp_ISR();                                                                     // call irmp ISR
  // call other timer interrupt routines...
  
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MAIN: main routine
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

int main (void)
{
	IRMP_DATA irmp_data;
	
	DDRB = ((1<<PINB0)|(1<<PINB1)|(1<<PINB2));										// Set Pin as output
	PORTB |=(1<<PINB3)|(1<<PINB4);
	
	uart_init();
	irmp_init ();                                                                   // initialize rc5
	timer_init ();
	
	PORTB |= (1 << PINB0);															// LED Run

	sei ();                                                                         // enable interrupts

	char text[50];
	for (;;)
	{
		if (irmp_get_data (&irmp_data))
		{
			PORTB |= (1 << PINB1);
			
			sprintf(text, "protocol: %d, adress: %x, command: %x\r\n",irmp_data.protocol,irmp_data.address,irmp_data.command);
			uart_puts(text);
			// ir signal decoded, do something here...
			// irmp_data.protocol is the protocol, see irmp.h
			// irmp_data.address is the address/manufacturer code of ir sender
			// irmp_data.command is the command code
		}
	}
}
